from django.http import HttpResponse
import py_eureka_client.eureka_client as eureka_client
import json


POST_REQUEST_TYPE="POST"
GET_REQUEST_TYPE="GET"

SERVICES_CONTEXT={
    "READ_FILE":["file_path_info"],
}

class Services():
    
    FILE_SERVICE_NAME="FileService"
    EDA_SERVICE_NAME="EDA-Service"
    DM_SERVICE_NAME="DataMinning-Service"
    DATABASE_SERVICE_NAME="DbController-Service"
    # DM_SERVICE="read_data/from_file/"
    READ_DF_FILE="read_data/from_file/"
    READ_MEDIA_FILE="read_data/from_media_from_service/"
    READ_DF_FILE_FROM_SERVICE="read_data/from_file_from_service/"
    READ_DF_FILE_RESPONSE_KEY="json_data"
    READ_DF_FILE_WITH_PAGINATION="read_data/from_file_with_pagination/"
    READ_DF_FILE_WITH_PAGINATION_RESPONSE_KEY="json_data"
    READ_DF_FILE_WITH_SUBCOLUMNS="read_data/from_file_with_subcolumns/"
    READ_DF_DATA_WITH_FROM_DB="read_data/from_db/"
    SAVE_DF_DATA_IN_FILE="read_data/save_file/"
    UPLOAD_FILES_FROM_SERVICE="read_data/upload_bulk_from_service/"
    SAVE_PICKLE="pickle/save_pickle/"
    FILE_KEY="file_guid"
    LOAD_PICKLE="pickle/load_pickle/"
    JSON_DATA_KEY="json_data"
    DATA_KEY="data"



class ServiceController():
    

    # def __init__(self,service_name=FILE_SERVICE) -> None:
    #     super.__init__()
    #     self._service_name=service_name
    #     self._service_tag=None
    #     self._request_type=None
    #     self._response_data=None
    #     self._response_error=None
    
    @staticmethod

    def CallEurekaService(service_name,service_url:str,data:dict,response_key:str,request_type=POST_REQUEST_TYPE):
        print("eureka call to ",service_url)
        # self._service_tag=service_url
        if service_url==Services.UPLOAD_FILES_FROM_SERVICE:

            serviceCallResults= json.loads(eureka_client.do_service(service_name,service_url,data=data,method=request_type))
        else:
            serviceCallResults= json.loads(eureka_client.do_service(service_name,service_url,data=json.dumps(data),method=request_type))
        if serviceCallResults["status"] == False:
                return False,{'message':serviceCallResults["message"],"code":serviceCallResults["code"]}
        if response_key is not None:
            return True,serviceCallResults[response_key]
        else:
            return True,serviceCallResults
    @staticmethod
    def SaveDfInFile(df,extension=".csv",sheet=None,template_guid=None):
        df_as_json=df.to_json(orient='records')
        serviceCallStatus,serviceCallResults= ServiceController.CallEurekaService(Services.FILE_SERVICE_NAME,Services.SAVE_DF_DATA_IN_FILE,data={"json_data":df_as_json,"extension":extension,"sheet":sheet,"template_guid":template_guid},response_key=Services.FILE_KEY)
        if not serviceCallStatus:
                return False,{'message':serviceCallResults["message"],"code":422}
        outfilename=serviceCallStatus
        return True,outfilename


    @staticmethod
    def ReturnErrorMessage(error_message=None):
            return HttpResponse(json.dumps({'status': False, 'message': error_message["message"]}),status=error_message["code"],
                                        content_type='application/json')



    # def CloseConnection():
    #     print("eureka call to ",service_url)
    #     # self._service_tag=service_url
    #     serviceCallResults= json.loads(eureka_client.do_service(self._service_name,service_url,data=data,method=request_type))
    #     if serviceCallResults["status"] == False:
    #             self._response_error={'message':serviceCallResults["message"],"code":serviceCallResults["code"]}
    #             return False
    #     if response_key is not None:
    #         self._response_data=serviceCallResults[response_key]
    #     else:
    #         self._response_data=serviceCallResults
    #     return True





import pandas as pd
import pathlib
# region Factorizing categorical columns
# This funcation will replace any categorical(object) feature with numbers for example (gender (F,M) 1,-1)
def factorize(_dataFrame):
    _copied = pd.DataFrame.copy(_dataFrame)
    for col in _dataFrame.columns:
        if _copied[col].dtype != 'int64' and _copied[col].dtype != 'float64':
            _copied[col], unique = pd.factorize(_copied[col])
    return _copied


# endregion Factorizing categorical columns


    
def GetTemplateGuidFromPath(path): 
    '''
    ----------
    Definition
    ----------
        Return template name from given path \n
        template name is the top level folder from given path

    ----------
    Example
    ----------
            path='123/usr/home/dataframe.xlsx'
            GetTemplateGuidFromPath(path) ----> 123

    ----------
    Parameters
    ----------
    files_list :list non empty files objects from user,``required``

    template_guid : str ,``required``
        parameter specify the folder that will be file saved in

    source : str ,``required``
        parameter return file content if source(reuqest) 'from_service'

    '''
    return pathlib.Path(path).parts[0]



#skweness  Enherafffn  https://www.analyticsvidhya.com/blog/2021/05/shape-of-data-skewness-and-kurtosis/
#kurtosis taffartoh   https://www.analyticsvidhya.com/blog/2021/05/shape-of-data-skewness-and-kurtosis/s

StatisticsDictionary = {
    #الالتواء
    "Skewness": {
        "Definition": "Skewness is a degree of asymmetry observed in a probability distribution that deviates from the symmetrical normal distribution (bell curve) in a given set of data.",
        "Example": "When data is symmetrically distributed, the left-hand side, and right-hand side, contain the same number of observations.mean=median=mode"
                   " (If the dataset has 100 values, then the left-hand side has 50 observations, and the right-hand side has 50 observations.)"
                   "But, what if not symmetrical distributed? That data is called asymmetrical data",
        "Types":["Negative skewed or left-skewed","Positive skewed or right-skewed"],
        "Hints":"If the skewness is between -0.5 & 0.5, the data are nearly symmetrical."
                "If the skewness is between -1 & -0.5 (negative skewed) or between 0.5 & 1(positive skewed), the data are slightly skewed"
                ".the skewness is lower than -1 (negative skewed) or greater than 1 (positive skewed), the data are extremely skewed."
        },

    "Kurtosis":{
        "Definition":"Kurtosis refers to the degree of presence of outliers in the distribution.",
        "Example":"",
        "Types":["Leptokurtic or heavy-tailed distribution (kurtosis more than normal distribution) >3","Mesokurtic (kurtosis same as the normal distribution).","Platykurtic or short-tailed distribution (kurtosis less than normal distribution)."],

        "Hints":"Leptokurtic (kurtosis > 3) Leptokurtic is having very long and skinny tails, which means there are more chances of 'outliers'. Positive values of kurtosis indicate that distribution is peaked and possesses thick tails. An extreme positive kurtosis indicates a distribution where more of the numbers are located in the tails of the distribution instead of around the mean."
                "Mesokurtic is the same as the normal distribution, which means kurtosis is near to 0. In Mesokurtic, distributions are moderate in breadth, and curves are a medium peaked height."
                "Platykurtic having a lower tail and stretched around center tails means most of the data points are present in high proximity with mean. A platykurtic distribution is flatter (less peaked) when compared with the normal distribution"
    },
    "Stability":{
        "Definition":"Measures how stable or constant this column is. The number of rows with the most frequent non-missing value divided by the total number of data rows with non-missing values."
    },
    "Missing":{
        "Definition":"The number of missing values in this column as a fraction of the total number of data rows"
    },
    "Outlier":{
        "Definition":"Extreme values that fall a long way outside of the other observations,its have negative effect on the model"
    },
    "Correlation (C)":{
        "Definition":"Measures the linear correlation between the data column and the target column. This quality bar is only available when the task is 'Predict'.",
                        "Example":"Low Correlation: a correlation of less than 0.01% indicates that this column is not likely to contribute to the predictions. While keeping such a column is not problematic, removing it may speed up the model building."
                                "High Correlation: a correlation of more than 40% may be an indicator for information you don't have at prediction time. In that case, you should remove this column. Sometimes, however, the prediction problem is simple, and you will get a better model when the column is included. Only you can decide."
    },
    "Idness (I)":{"Definition":" measures the degree to which this Attribute resembles an ID. The number of different values for the Attribute divided by the number of data rows."},
    'Mean Absolute Error(MAE)':{'Definition':'MAE is a very simple metric which calculates the absolute difference between actual and predicted values then sum all the errors and divide them by a total number of observations,and we aim to get the minimum MAE',
                                   'Examples' :['most Robust to outliers','Use MAE if you do not want to penalize large prediction errors.','common metrics used to measure accuracy for continuous variables']
                                },
    'Mean Squared Error (MSE)' :{'Definition':'It represents the squared distance between actual and predicted values. we perform squared to avoid the cancellation of negative terms and it is the benefit of MSE.',
                                'Examples': ['not Robust to outliers',
                                            'it is always a positive value',
                                            '0 means the model is perfect',
                                            'The smaller the mean squared error ,The closer you are to finding the line of best fit. Depending on your data',
                                            'When a model has no error, the MSE equals zero',
                                            'As model error increases, MSE value increases',
                                            'MSE gives larger penalization to big prediction error by square it while MAE treats all errors the same.',
                                            'For an ideal model, RMSE/MAE=0',
                                            ]    
                            },
    'Root Mean Squared Error(RMSE)':{'Definition':'It represents the root squared distance between actual and predicted values. we perform squared to avoid the cancellation of negative terms and it is the benefit of MSE.',
                                    'Examples': ['The RMSE will always be larger or equal to the MAE',
                                                'If the RMSE=MAE, then all the errors are of the same magnitude',
                                                'Both the MAE and RMSE can range from 0 to ∞.'
                                                'If RMSE>MAE, then there is variation in the errors.',
                                                'If the RMSE-MAE difference was isnt large enough so no need to indicate the presence of very large errors.',
                                                'one distinct advantage of RMSE over MAE is that RMSE avoids the use of taking the absolute value, which is undesirable in many mathematical calculations ',
]
                                },
}

##Generalized Linear Model: generalization of linear regression models
# Deep Learning: multi-level neural network for learning non-linear relationships
# Decision Tree: finds simple tree-like models which are easy to understand
# Random Forest: ensemble of multiple randomized trees
# Gradient Boosted Trees: powerful but complex model using ensembles of decision trees
# Support Vector Machine: powerful but relatively fast model, especially for non-linear relationships




import code
from django.http import JsonResponse

from django_numpy_json_encoder.numpy_encoder import NumpyJSONEncoder

class EncodedJsonResponse_success(JsonResponse):
    """
    An HTTP response class that consumes data to be serialized to JSON and encode numpy types like float32,int32 to float64,int64 to be accepted in json .

    :param data: Data to be dumped into json. By default only ``dict`` objects
      are allowed to be passed due to a security flaw before EcmaScript 5. See
      the ``safe`` parameter for more information.
    :param safe: Controls if only ``dict`` objects may be serialized. Defaults
      to ``True``

    """
    def __init__(self,data,safe=True,**kwargs) -> None:
        super().__init__(data=data,encoder=NumpyJSONEncoder,safe=safe, **kwargs)

class EncodedJsonResponse_failure(JsonResponse):
    """
    An HTTP response class that consumes error response and if error have int32,float32 will be encoded in int64,float64

    :param data: Data to be dumped into json. By default only ``dict`` objects
      are allowed to be passed due to a security flaw before EcmaScript 5. See
      the ``safe`` parameter for more information.
    :param safe: Controls if only ``dict`` objects may be serialized. Defaults
      to ``True``

    """
    def __init__(self,error_message,safe=True,**kwargs) -> None:
        super().__init__(content={"message":error_message["message"]},status=error_message["code"],encoder=NumpyJSONEncoder, **kwargs)

    


from django.http.response import HttpResponse
import numpy as np
import json
import base64

from .processing import GetTemplateGuidFromPath
from mka_eureka_microservice_controllers.shared_scripts import mka_requests
from .base64_converter import Base64Converter
from .services_managment import ServiceController,Services

class ParamsMapper(object):
    DICTIONARY_LIST_TYPE="DictionaryList"

    _key_for_all_requests={

            "file_path_info": {"type":DICTIONARY_LIST_TYPE,"has_default":True,"default":[]},
            "keywords_count": {"type":int,"has_default":True,"default":4},
            "keywords_algorithms": {"type":list,"has_default":True,"default":["YAKE"]},
            "sentiment_option": {"type":dict,"has_default":True,"default":{"name": "Text", "value": None}},
            "cleaning_codes": {"type":dict,"has_default":True,"default":{}},
            "forbidden_words": {"type":dict,"has_default":True,"default":{}},
            "content_language": {"type":str,"has_default":True,"default":"en"},
            "selected_analysis_type": {"type":list,"has_default":False},
            "summary_algorithm": {"type":list,"has_default":True,"default": ["LEX_RANK"]},
            "minimizing_approach": {"type":dict,"has_default":True,"default": {'action': 'numberOfBestSentences', 'value': 5}},
    }

    @classmethod

    def GetParamsMappingsDictionary(cls,expected_keys_list):
        return {key:ParamsMapper._key_for_all_requests.get(key) for key in expected_keys_list}
class RequestParser():
    DICTIONARY_LIST_TYPE=ParamsMapper.DICTIONARY_LIST_TYPE
    def __init__(self,req,expected_keys_list) -> None:
        # self._request_files=req.FILES.getlist('files')
        self._passed_request=req
        self._expected_keys=ParamsMapper.GetParamsMappingsDictionary(expected_keys_list)
        self._request_dict=dict()
        self._error=dict()
        

    def ParseRequest(self):
        self._request_dict=self._passed_request.data

        requestContentType=self._passed_request.META['CONTENT_TYPE']
        if requestContentType=="application/json":
            stauts,response=self._JsonRequestLoader()
        else:
            stauts,response=self._FormDataRequestLoader()
        

        if not stauts:
            self._error=response
        else:
            self._request_dict=response

        return stauts


    def _JsonRequestLoader(self):

        status,response=self._ValidateRequest()
        return status,response
    

    # return body



    def _FormDataRequestLoader(self):

        self._passed_request.data._mutable=True

        # self._passed_request.query_params.gitlist("file_path_info")
        foundKeys=np.intersect1d(list(self._request_dict.keys()),list(self._expected_keys.keys()))
        self._request_dict=self._passed_request.data.copy()
        for key in foundKeys:
            properties=self._expected_keys[key]
            if properties["type"]==self.DICTIONARY_LIST_TYPE :
                keyValues=self._passed_request.data.getlist(key)
                print(keyValues)
                if isinstance(keyValues,str):
                    keyValues=[json.loads(keyValues)]
                else:
                    for dict_str in keyValues:
                         dict_str=json.loads(dict_str)
                self._request_dict[key]=keyValues
                print(self._request_dict[key])
            elif properties["type"]==list:
                self._request_dict[key]=self._request_dict.getlist(key)
            elif properties["type"]==dict:
                self._request_dict[key]=json.loads(self._request_dict[key])
            elif properties["type"]==bool:
                self._request_dict[key]=json.loads(self._request_dict[key].lower())
            else:
                self._request_dict[key]=properties["type"](self._request_dict[key])
                continue
            # if isinstance(self._request_dict[key]["type"]
        # for key,value in passedFormData.items():
        #     self.passedFormData[key]=str(value) if self._expected_keys[key]=="String" else int(value) if self._expected_keys[key]=="Number" else float(value) if self._expected_keys[key]=="Decimal" else str(value) if self._expected_keys[key]=="String"
        
        status,validation_response=self._ValidateRequest()
        self._passed_request.data._mutable=False
        return status,validation_response


    def _ValidateRequest(self):
        requestkeys=self._request_dict.keys()
        notFoundKeysInRequest=[]
        for key,value in self._expected_keys.items():
            if key in requestkeys:
                if value["type"]==self.DICTIONARY_LIST_TYPE:
                    try:
                        self._request_dict[key]=self._request_dict[key]
                    except:
                        example=value.get("example")
                        notFoundKeysInRequest.append(f"{key}:value like {example}")
                    continue

                elif value["type"]==dict:
                    if isinstance(self._request_dict[key],dict):
                        continue
                    else:
                        notFoundKeysInRequest.append(f"{key}:should be map key:value")
                        continue
                elif value["type"]==list:
                    if not isinstance(self._request_dict[key],list):
                        notFoundKeysInRequest.append(f"{key}:should be list")
                    if "values" in value.keys():
                        for keyValue in self._request_dict[key]:
                            if keyValue not in value["values"] :
                                notFoundKeysInRequest.append(f"{key} values should be in {'or'.join(i for i in value['values'])}")
                                break
                elif value["type"]==dict:
                    if not isinstance(self._request_dict[key],dict):
                        notFoundKeysInRequest.append(f"{key}:should be map key:value")

                elif value["type"]==bool:
                    if not isinstance(self._request_dict[key],dict):
                        notFoundKeysInRequest.append(f"{key}:should be boolean True,False")
                elif value["type"]==any:
                    continue
                else:
                    print(self._request_dict[key],value["type"])
                    if not isinstance(self._request_dict[key],value["type"]):
                        notFoundKeysInRequest.append(f"{key}:should be {'number' if value['type']==int else 'decimal' if value['type']==float else 'text'}")
                    self._request_dict[key]=value["type"](self._request_dict[key])
                    continue
            elif value["has_default"]:
                self._request_dict[key]=value["default"]
            else:
                notFoundKeysInRequest.append(key)

        return True if len(notFoundKeysInRequest)==0 else False,self._request_dict if len(notFoundKeysInRequest)==0 else notFoundKeysInRequest

    def ReadListFiles(self):
        passedFiles=self._passed_request.FILES.getlist("files",[])
        passedFilesData=[]
        if passedFiles!=[]:
            status,response=ServiceController.CallEurekaService(Services.FILE_SERVICE_NAME,Services.UPLOAD_FILES_FROM_SERVICE,self._passed_request,Services.DATA_KEY)
            if not status:
                return response

            passedFilesData.extend(response)


        if self._request_dict["file_path_info"] is not None and self._request_dict["file_path_info"] !=[] :

            status,response=ServiceController.CallEurekaService(Services.FILE_SERVICE_NAME,Services.READ_DF_FILE_FROM_SERVICE,{"file_path_info":self._request_dict["file_path_info"]},Services.DATA_KEY)
            if not status:
                return response
    
            passedFilesData.extend(response)

        for file in passedFilesData:
            file.update({"template_guid":GetTemplateGuidFromPath["file_path"]})
        return passedFiles

    def ReadFile(self,file_path_info={}):
        
        passedFiles=None if file_path_info=={} else self._passed_request.FILES.get("files",None)
        passedFilesData=[]
        if passedFiles is not None:
            status,response=ServiceController.CallEurekaService(Services.FILE_SERVICE_NAME,Services.FILE_SERVICE_NAME,Services.UPLOAD_FILES_FROM_SERVICE,{"files":base64.b64encode(self._passed_request.FILES.get("files").read()).decode()},Services.DATA_KEY)
            if not status:
                return response
            
            response[0].update({"template_guid":GetTemplateGuidFromPath(response[0]["file_path"])})
            return response[0]


        fileToRead=file_path_info["file_path_info"] if file_path_info!={} else self._request_dict["file_path_info"] 
        status,response=ServiceController.CallEurekaService(Services.FILE_SERVICE_NAME,Services.FILE_SERVICE_NAME,Services.READ_DF_FILE,{"file_path_info":fileToRead[0:1] if isinstance(fileToRead,list) else [fileToRead]},Services.DATA_KEY)
        if not status:
            return response
        
        response[0].update({"template_guid":GetTemplateGuidFromPath(response[0]["file_path"])})
        return response[0]

    def ReadMediaFiles(self):
        
        passedFiles=self._passed_request.FILES.get("files",None)

        passedFilesData=[]
        if passedFiles is not None:
            status,response=ServiceController.CallEurekaService(Services.FILE_SERVICE_NAME,Services.UPLOAD_FILES_FROM_SERVICE,self._passed_request.FILES.get("files"),Services.DATA_KEY)
            if not status:

                return False,response
            passedFilesData.extend(response)

        if self._request_dict["file_path_info"] is not None or self._request_dict["file_path_info"] !=[] :
            status,response=ServiceController.CallEurekaService(Services.FILE_SERVICE_NAME,Services.READ_MEDIA_FILE,{"file_path_info":self._request_dict["file_path_info"]},Services.DATA_KEY)
            if not status:

                return False,response
            if not status:
                return False,response
            
            passedFilesData.extend(response)

        return True,passedFilesData 




    @staticmethod
    def SaveDfInFile(cls,df,extension=".csv",sheet=None,template_guid=None):
        df_as_json=df.to_json(orient='records')
        serviceCallStatus,serviceCallResults= ServiceController.CallEurekaService(Services.FILE_SERVICE_NAME,Services.SAVE_DF_DATA_IN_FILE,data={"json_data":df_as_json,"extension":extension,"sheet":sheet,"template_guid":template_guid},response_key=Services.FILE_KEY)
        if not serviceCallStatus:
                return False,{'message':serviceCallResults["message"],"code":422}
        outfilename=serviceCallStatus
        return True,outfilename

    def ReadPickle(self,pickle_path:str,pickle_type="model"):
        serviceController=ServiceController(Services.FILE_SERVICE_NAME)
        serviceCallResults= serviceController.CallEurekaService(Services.LOAD_PICKLE,data={"pickle_path":pickle_path,"pickle_type":pickle_type},response_key=Services.DATA_KEY)

        if not serviceCallResults:
                return False,{'message':serviceCallResults["message"],"code":422}
        readResult=serviceController._response_data
        readResult[pickle_path]=Base64Converter.B64ToPickle(readResult[str(pickle_path)])
        if "TransformersObjects" in readResult.keys() and readResult["TransformersObjects"]!={}:
            transformerObjects=readResult["TransformersObjects"]
            for pickle_name,pickle_b64 in transformerObjects.items():
                transformerObjects[pickle_name]=Base64Converter.B64ToPickle(pickle_b64)
        return True,readResult
    def ReturnErrorMessage(self,error_message=None):
        if not error_message:
            return HttpResponse(json.dumps({'status': False, 'message': ",".join(i for i in self._error)}), status=422,
                                        content_type='application/json')
        else :
            return HttpResponse(json.dumps({'status': False, 'message': error_message["message"]}),status=error_message["code"],
                                        content_type='application/json')








import setuptools
from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="mka-eureka-microservice-controllers",
    version="0.1.0",
    author="Adnan Aji",
    author_email="",
    description="A package contains scripts to easily connection between connected django microservice with eureka",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="",
    project_urls={
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License"
    ],
    package_dir={'':"src"},
    packages=find_packages("src"),
    python_requires=">=3.6",
    entry_points={
                        'console_scripts': [
                                'hwpypcmd=hwpyp.mypy:sayHello',
                        ]
                }
)


import base64
import pickle

class Base64Converter(object):
    

    @classmethod
    def B64ToPickle(b64_pickle):
        pickleData=base64.b64decode(b64_pickle)
        pickleData=pickle.loads(pickleData)
        return pickleData
    @classmethod
    def PickleToB64(pickle_data):
        b64Pickle=pickle.dumps(pickle_data)
        b64Pickle=f"""{base64.b64encode(b64Pickle).decode()}"""
        return b64Pickle
    @classmethod
    def B64ToHtml(b64_html):
        html_str=base64.b64decode(b64_html)
        return html_str
    @classmethod
    def HtmlToB64(html_str):
        b64_html=base64.b64encode(html_str.encode('ascii')).decode()
        return b64_html

    # def FileUploadToB64(html_str):
    #     b64_html=base64.b64encode(html_str.encode('ascii')).decode()
    #     return b64_html

